% mat1401X.m
% mfile to demonstrate use of matced32 routines
% for details of 1401 command strings see the 1401 Programmer's handbook
%
% this example assumes that matced32 is installed on the matlab
% path and that use1432.dll is in the system directory
%
% reads channels 0 and 1 and extracts the data for display
% with menu option of waiting for a trigger pulse
% uses ~X version of Open, Reset and Close that do not echo
% to the screen

% by JG Colebatch July 2004
% note that polling may lock up the PC until data has been collected
% unless drawnow or pause(n) is called
% drawnow is likely to give the fastest response to the 1401.
% June 2004: allow option of trigger
% May 2006: use cedLdX (version 2)
% Dec 2006: sample faster  - 5kHz: 
%          for ADCMEM need to sample at chan x desired rate = 10 kHz
%          for ADCBST need to just set desired sample rate = 5kHz
%          works with v2 of MATCED32.dll
% 15.02.2007: modify to avoid sampling bug on Power1401, needs v3 of dll
% 13.02.2008: Alter case of MATCED32.DLL for Matlab7
% 03.11.2009: modify default 1410 location to C:\1401Lang\, consistent with
%               CED download.
% 11.11.2009: call WorkingSet to avoid -544 error


U14TYPEPOWER=3;  % Power1401  
yScale=5/32768; % results in Volts
res=MATCED32('cedOpenX',0);  % v3+ can specify 1401
if (res < 0)
   disp(['1401 not opened, error number ' int2str(res)]);
else %opened OK  
typeOf1401=MATCED32('cedTypeOf1401');  % see below
res=MATCED32('cedResetX');
res=0;
% uncomment this next line, if you get a -544 error later
% res=MATCED32('cedWorkingSet',400,4000);
if (res > 0)
    disp('error with call to cedWorkingSet - try commenting it out');
    return
end    
% next line needs v3 or later of matced32.dll
MATCED32('cedLdX','C:\1401Lang\','ADCMEM','ADCBST');
% collect 100 pts at 5000 Hz from both channels 0 and 1 = 400 bytes,
% and implies a overall sampling rate of 10000 Hz for ADCMEM = 16x25
% but just set desired sample rate for ADCBST (5000Hz) = 32x25 (H clock)
if typeOf1401==U14TYPEPOWER
  MATCED32('cedSendString','ADCBST,T,0,100;');  % limit burst rate on Power14
end
ktrig=input('Await trigger pulse (y=yes)? ','s');
if ktrig =='y'
   % for ADCMEM, next line should use ... HT,16,25;
   disp('Waiting for trigger (on Trigger input)');
   MATCED32('cedSendString','ADCBST,I,2,0,400,0 1,1,HT,32,25;');
else % default option
   MATCED32('cedSendString','ADCBST,I,2,0,400,0 1,1,H,32,25;');
end   
res=-1;
% 1401 polling loop, note pause which allows other processing
while res ~=0
   MATCED32('cedSendString','ADCBST,?;');
   res=eval(MATCED32('cedGetString'));
   % pause(0.1);  % this is an alternative to drawnow
   drawnow; % flushes the event queue
end
% now get the data, no. of words not bytes
data=MATCED32('cedToHost',200,0);
res=MATCED32('cedCloseX');
% extract the channel data
chan1samples=1:2:199;
chan2samples=chan1samples+1;
y1=data(chan1samples);
y2=data(chan2samples);
time=1:100;
time=time/5000; % in seconds
subplot(2,1,1), plot(time, yScale*y1);
title('1401 sampled data, channel 1');
xlabel('Time (s)');
ylabel('Input (Volts)');
subplot(2,1,2), plot(time, yScale*y2);
title('1401 sampled data, channel 2');
xlabel('Time (s)');
ylabel('Input (Volts)');
end